import os, subprocess

'''A build system has three methods: "configure" (with optionak "force" parameter), "build" and "install"'''

# Compile, build, and install cmake projects:
class CMake:
	def __init__(self, sourceFolder, buildFolder, module, config):
		self.sourceFolder = os.path.abspath(sourceFolder)
		self.buildFolder = os.path.abspath(buildFolder)
		self.installDir = config['installDir']
		self.buildType = config['buildType']
		self.jobs = config['jobs']
		self.buildCmdPrefix = config['buildCmdPrefix']
		self.installCmdPrefix = config['installCmdPrefix']
		self.module = module
	
	def build(self, reconfigure=False):
		# Make sure we have a build directory
		if not os.path.exists(self.buildFolder): os.makedirs(self.buildFolder)
		os.chdir(self.buildFolder)
		# In case of reconfiguration, delete cache file if it exists
		cacheFile = 'CMakeCache.txt'
		if os.path.exists(cacheFile) and reconfigure: os.remove(cacheFile)
		# Run cmake
		os.putenv('PKG_CONFIG_PATH', os.path.join(self.installDir, 'lib', 'pkgconfig')) # I found no way to do this within cmake
		subprocess.check_call(['cmake', self.sourceFolder, '-DCMAKE_BUILD_TYPE='+self.buildType,
			'-DCMAKE_INSTALL_PREFIX='+self.installDir]+self.module.get('cmakeParameters', []))
		os.unsetenv('PKG_CONFIG_PATH')
		# run compilation
		subprocess.check_call(self.buildCmdPrefix + ['make', '-j'+str(self.jobs)])
		# run installation
		subprocess.check_call(self.installCmdPrefix + ['make', 'install'])

# if auto-debuild is available, provide a wrapper for it
try:
	import auto_debuild
	class AutoDebuild:
		def __init__(self, sourceFolder, buildFolder, module, vcs, config):
			self.sourceFolder = os.path.abspath(sourceFolder)
			self.buildFolder = os.path.abspath(buildFolder)
			self.jobs = config['jobs']
			self.debDir = os.path.abspath(config['debDir'])
			self.debName = config['debName']
			self.debEMail = config['debEMail']
			self.module = module
			self.vcs = vcs

		def build(self, reconfigure=False): # reconfigure is ignored (we always do a reconfiguration)
			# create auto-debuild configuration
			autoDebuildConfig = {
				'sourceName': self.module['name'],
				'buildSystem': self.module['buildSystem'],
				'debDir': self.debDir,
				'buildDir': self.buildFolder,
				'name': self.debName,
				'email': self.debEMail,
				'parallelJobs': self.jobs,
				'version': self.vcs.version()
			}
			if autoDebuildConfig['version'] is None:
				raise Exception("VCS did not provide us with a proper version number, please fix this")
			# copy some more optional configuration
			for option in ('dbgPackage', 'section', 'binarySkipFiles', 'binaryInstallFiles',
					'buildDepends', 'binaryDepends', 'binaryRecommends', 'binaryProvides',
					'cmakeParameters', 'automakeParameters'):
				if option in self.module:
					autoDebuildConfig[option] = self.module[option]
			# create Debian files
			os.chdir(self.sourceFolder)
			files = auto_debuild.createDebianFiles(autoDebuildConfig)
			# build package(s)
			auto_debuild.buildDebianPackage(autoDebuildConfig)
			# install package(s)
			subprocess.check_call(['sudo', 'dpkg', '--install'] + files)

except ImportError:
	print "auto_debuild not found, disabling auto-debuild system"
	pass
